using System;
using System.Collections;
using Microsoft.PointOfService;
using Microsoft.PointOfService.BasicServiceObjects;

namespace Opos.PointOfService.BasicShimServiceObjects
{
	/// <summary>
	/// It is an explanation of the outline of CashChangerShimBasic. 
	/// </summary>
	public abstract class CashChangerShimBasic : CashChangerBasic
	{
		public CashChangerShimBasic()
		{
			// 
			// TODO: Please add the constructor logic here.
			//
		}

		#region POS for .NET 1.0
		#region in/out parameter

		#region DirectIO method
		public sealed override DirectIOData DirectIO(int command, int data, object obj)
		{
			DirectIO(command, ref data, ref obj);

			return new DirectIOData(data, obj);
		}

		protected abstract void DirectIO(int command, ref int data, ref object obj);
		#endregion

		#region ReadCashCounts method
		public sealed override CashCounts ReadCashCounts()
		{
			string StrCashCounts = null;
			bool discrepancy = false;
			ReadCashCounts(ref StrCashCounts, ref discrepancy);

			if (StrCashCounts == null || StrCashCounts.Length == 0)
				return new CashCounts(new CashCount[0], discrepancy);

			ArrayList ar = new ArrayList();
			string[] StrCashCountsArray1 = StrCashCounts.Split(new char[1]{';'});
			if (StrCashCountsArray1.Length <= 2)
			{
				for (int i = 0; i < StrCashCountsArray1.Length; i ++)
				{
					if (StrCashCountsArray1[i] == null || StrCashCountsArray1[i].Length == 0)
						continue;
					string[] StrCashCountsArray2 = StrCashCountsArray1[i].Split(new char[1]{','});
					for (int j = 0; j < StrCashCountsArray2.Length; j ++)
					{
						if (StrCashCountsArray2[j] == null || StrCashCountsArray2[j].Length == 0)
							continue;
						string[] StrCashCountsArray3 = StrCashCountsArray2[j].Split(new char[1]{':'});
						if (StrCashCountsArray3.Length != 2)
							continue;
						ar.Add(new CashCount(
							i == 0 ? CashCountType.Coin : CashCountType.Bill,
							int.Parse(StrCashCountsArray3[0]),
							int.Parse(StrCashCountsArray3[1])));
					}
				}
			}

			CashCount[] counts = new CashCount[ar.Count];
			for (int k = 0; k < ar.Count; k ++)
			{
				counts[k] = (CashCount)ar[k];
			}

			return new CashCounts(counts, discrepancy);
		}

		protected abstract void ReadCashCounts(ref string cashCounts, ref bool discrepancy);
		#endregion

		#endregion

		#region Add methods for UPOS I/F

		#region ResetStatistic/ResetStatistics method
		public sealed override void ResetStatistic(string statistic)
		{
			base.VerifyState(true, true);
			if(!base.CapStatisticsReporting)
			{
				throw new PosControlException("The device does not support statistics.", ErrorCode.Illegal);
			}
			if(!base.CapUpdateStatistics)
			{
				throw new PosControlException("The device does not support updating statistics.", ErrorCode.Illegal);
			}
			if(statistic == null)
			{
				throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}
			ResetStatistics(statistic);
		}

		public sealed override void ResetStatistics()
		{
			base.VerifyState(true, true);
			if(!base.CapStatisticsReporting)
			{
				throw new PosControlException("The device does not support statistics.", ErrorCode.Illegal);
			}
			if(!base.CapUpdateStatistics)
			{
				throw new PosControlException("The device does not support updating statistics.", ErrorCode.Illegal);
			}
			ResetStatistics("");
		}

		public sealed override void ResetStatistics(StatisticCategories statistics)
		{
			base.VerifyState(true, true);
			if(!base.CapStatisticsReporting)
			{
				throw new PosControlException("The device does not support statistics.", ErrorCode.Illegal);
			}
			if(!base.CapUpdateStatistics)
			{
				throw new PosControlException("The device does not support updating statistics.", ErrorCode.Illegal);
			}
			string param = "";
			switch(statistics)
			{
				case StatisticCategories.All:
					param = "";
					break;
				case StatisticCategories.Manufacturer:
					param = "M_";
					break;
				case StatisticCategories.Upos:
					param = "U_";
					break;
				default:
					throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}
			ResetStatistics(param);
		}

		public sealed override void ResetStatistics(string[] statistics)
		{
			base.VerifyState(true, true);
			if(!base.CapStatisticsReporting)
			{
				throw new PosControlException("The device does not support statistics.", ErrorCode.Illegal);
			}
			if(!base.CapUpdateStatistics)
			{
				throw new PosControlException("The device does not support updating statistics.", ErrorCode.Illegal);
			}
			if(statistics == null || statistics.Length == 0)
			{
				throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}
			if(Array.IndexOf(statistics, null) != -1)
			{
				throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}

			string param = "";
			param = String.Join(",", statistics);

			ResetStatistics(param);
		}

		protected abstract void ResetStatistics(string statistics);
		#endregion

		#region RetrieveStatistic/RetrieveStatistics method
		public sealed override string RetrieveStatistic(string statistic)
		{
			base.VerifyState(true, true);
			if(!base.CapStatisticsReporting)
			{
				throw new PosControlException("The device does not support statistics.", ErrorCode.Illegal);
			}
			if(statistic == null)
			{
				throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}
			RetrieveStatistics(ref statistic);

			return statistic;
		}
		public sealed override string RetrieveStatistics()
		{
			base.VerifyState(true, true);
			if(!base.CapStatisticsReporting)
			{
				throw new PosControlException("The device does not support statistics.", ErrorCode.Illegal);
			}

			string param = "";
			RetrieveStatistics(ref param);

			return param;
		}
		public sealed override string RetrieveStatistics(StatisticCategories statistics)
		{
			base.VerifyState(true, true);
			if(!base.CapStatisticsReporting)
			{
				throw new PosControlException("The device does not support statistics.", ErrorCode.Illegal);
			}

			string param = "";
			switch(statistics)
			{
				case StatisticCategories.All:
					param = "";
					break;
				case StatisticCategories.Manufacturer:
					param = "M_";
					break;
				case StatisticCategories.Upos:
					param = "U_";
					break;
				default:
					throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}
			RetrieveStatistics(ref param);

			return param;
		}
		public sealed override string RetrieveStatistics(string[] statistics)
		{
			base.VerifyState(true, true);
			if(!base.CapStatisticsReporting)
			{
				throw new PosControlException("The device does not support statistics.", ErrorCode.Illegal);
			}
			if(statistics == null || statistics.Length == 0)
			{
				throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}
			if(Array.IndexOf(statistics, null) != -1)
			{
				throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}

			string param = "";
			param = String.Join(",", statistics);

			RetrieveStatistics(ref param);
			return param;
		}

		protected abstract void RetrieveStatistics(ref string statistics);
		#endregion

		#region UpdateStatistic/UpdateStatistics method
		public sealed override void UpdateStatistic(string name, object value)
		{
			base.VerifyState(true, true);
			if(!base.CapStatisticsReporting)
			{
				throw new PosControlException("The device does not support statistics.", ErrorCode.Illegal);
			}
			if(!base.CapUpdateStatistics)
			{
				throw new PosControlException("The device does not support updating statistics.", ErrorCode.Illegal);
			}
			if(name == null || value == null)
			{
				throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}

			string param = name + "=" + value.ToString();
			UpdateStatistics(param);
		}

		public sealed override void UpdateStatistics(Statistic[] statistics)
		{
			base.VerifyState(true, true);
			if(!base.CapStatisticsReporting)
			{
				throw new PosControlException("The device does not support statistics.", ErrorCode.Illegal);
			}
			if(!base.CapUpdateStatistics)
			{
				throw new PosControlException("The device does not support updating statistics.", ErrorCode.Illegal);
			}
			if(statistics == null || statistics.Length == 0)
			{
				throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}

			string param = "";
			string[] arrayParam = new string[statistics.Length];
			int count = 0;

			foreach(Statistic statistic in statistics)
			{
				if(statistic == null || statistic.Name == null || statistic.Value == null)
				{
					throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
				}
				arrayParam[count++] = (statistic.Name + "=" + statistic.Value.ToString());
			}
			param = String.Join(",", arrayParam);

			UpdateStatistics(param);
		}

		public sealed override void UpdateStatistics(StatisticCategories statistics, object value)
		{
			base.VerifyState(true, true);
			if(!base.CapStatisticsReporting)
			{
				throw new PosControlException("The device does not support statistics.", ErrorCode.Illegal);
			}
			if(!base.CapUpdateStatistics)
			{
				throw new PosControlException("The device does not support updating statistics.", ErrorCode.Illegal);
			}

			string param = "";
			switch(statistics)
			{
				case StatisticCategories.All:
					param = "=";
					break;
				case StatisticCategories.Manufacturer:
					param = "M_=";
					break;
				case StatisticCategories.Upos:
					param = "U_=";
					break;
				default:
					throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}
			if(value == null)
			{
				throw new PosControlException("An invalid parameter value was specified.", ErrorCode.Illegal);
			}
			param += value.ToString();

			UpdateStatistics(param);
		}

		protected abstract void UpdateStatistics(string statistics);
		#endregion

		#endregion

		#region replace const to enum

		#endregion
		#endregion

		#region POS for .NET 1.1
		#region in/out parameter

		#region CompareFirmwareVersion method
		public sealed override CompareFirmwareResult CompareFirmwareVersion(string firmwareFileName)
		{
			int result = 0;
			CompareFirmwareVersion(firmwareFileName, ref result);
			return (CompareFirmwareResult)result;
		}
		protected virtual void CompareFirmwareVersion(string firmwareFileName, ref int result)
		{
			base.VerifyState(true, true);

			if(!base.CapCompareFirmwareVersion)
			{
				throw new PosControlException("The device does not support the requested functionality.", ErrorCode.Illegal);
			}
			throw new NotImplementedException();
		}
		#endregion

		#endregion


		#region Add methods for UPOS I/F

		#endregion


		#region replace const to enum

		#endregion
		#endregion
	}
}
